package fr.mrdev023.vulkan_java.vk;

import org.lwjgl.glfw.GLFWVulkan;
import org.lwjgl.system.MemoryStack;
import org.lwjgl.vulkan.KHRSurface;
import org.tinylog.Logger;

import static fr.mrdev023.vulkan_java.vk.VulkanError.vkCheck;

import java.nio.LongBuffer;

public class Surface {

    private final PhysicalDevice physicalDevice;
    private final long vkSurface;

    public Surface(PhysicalDevice physicalDevice, long windowHandle) throws VulkanError {
        this.physicalDevice = physicalDevice;

        try (MemoryStack stack = MemoryStack.stackPush()) {
            LongBuffer pSurface = stack.mallocLong(1);

            var result = GLFWVulkan.glfwCreateWindowSurface(this.physicalDevice.getVkPhysicalDevice().getInstance(),
                    windowHandle,
                    null, pSurface);
            vkCheck(result, "Failed to create Vulkan Surface");

            vkSurface = pSurface.get(0);
        }

        Logger.debug("Vulkan surface created");
    }

    public void destroy() {
        KHRSurface.vkDestroySurfaceKHR(physicalDevice.getVkPhysicalDevice().getInstance(), vkSurface, null);
        Logger.debug("Vulkan surface destroyed");
    }

    public long getVkSurface() {
        return vkSurface;
    }
}
