package fr.mrdev023.vulkan_java.window;

import static org.lwjgl.glfw.GLFW.*;
import static org.lwjgl.system.MemoryUtil.*;

import java.nio.*;

import org.lwjgl.*;
import org.lwjgl.glfw.*;

public class Display {

    private static DisplayMode displayMode;
    private static String TITLE = "";
    private static long window;
    private static boolean hasResized = false;

    public static void create(String title, int width, int height) {
        if (!glfwInit())
            throw new IllegalStateException("Unable to initialize GLFW");

        TITLE = title;
        displayMode = new DisplayMode(width, height);

        // Configure GLFW for Vulkan
        glfwDefaultWindowHints();
        glfwWindowHint(GLFW_CLIENT_API, GLFW_NO_API); // Tell GLFW not to create an OpenGL context
        glfwWindowHint(GLFW_VISIBLE, GLFW_FALSE);
        glfwWindowHint(GLFW_RESIZABLE, GLFW_TRUE);

        // Create the window
        window = glfwCreateWindow(displayMode.getWidth(), displayMode.getHeight(), TITLE, NULL, NULL);
        if (window == NULL)
            throw new RuntimeException("Failed to create the GLFW window");

        // Make the window visible
        glfwShowWindow(window);
    }

    public static void setMouseGrabbed(boolean a) {
        if (a) {
            glfwSetInputMode(window, GLFW_CURSOR, GLFW_CURSOR_DISABLED);
        } else {
            glfwSetInputMode(window, GLFW_CURSOR, GLFW_CURSOR_NORMAL);
        }
    }

    public static void setVSync(boolean a) {
        if (a)
            glfwSwapInterval(1);
        else
            glfwSwapInterval(0);
    }

    public static void setSample(int sample) {
        glfwWindowHint(GLFW_SAMPLES, sample);
    }

    public static void setResizable(boolean a) {
        if (a)
            glfwWindowHint(GLFW_RESIZABLE, GLFW_TRUE);
        else
            glfwWindowHint(GLFW_RESIZABLE, GLFW_FALSE);
    }

    public static void setTitle(String title) {
        TITLE = title;
        glfwSetWindowTitle(window, TITLE);
    }

    public static String getTitle() {
        return TITLE;
    }

    public static boolean wasResized() {
        IntBuffer w = BufferUtils.createIntBuffer(1);
        IntBuffer h = BufferUtils.createIntBuffer(1);
        glfwGetWindowSize(window, w, h);
        int width = w.get(0);
        int height = h.get(0);

        if (Display.getDisplayMode().getWidth() != width || Display.getDisplayMode().getHeight() != height
                || hasResized) {
            setDisplayMode(new DisplayMode(width, height));
            hasResized = false;
            return true;
        } else {
            return false;
        }
    }

    public static void printMonitorsInfo() {
        PointerBuffer monitors = glfwGetMonitors();
        GLFWVidMode m;
        if (monitors == null) {
            System.out.println("No monitor detected !");
            return;
        }
        for (int i = 0; i < monitors.capacity(); i++) {
            m = glfwGetVideoMode(monitors.get(i));
            if (m == null)
                continue;

            System.out.println(glfwGetMonitorName(monitors.get(i)) + "(" + i + ") : " + m.width() + "x" + m.height()
                    + ":" + m.refreshRate() + "Hz");
        }

    }

    public static boolean isCloseRequested() {
        return glfwWindowShouldClose(window);
    }

    public static void updateEvent() {
        glfwPollEvents();
    }

    public static void updateFrame() {
        glfwSwapBuffers(window);
    }

    public static DisplayMode getDisplayMode() {
        return displayMode;
    }

    public static void setDisplayMode(DisplayMode displayMode) {
        if (Display.displayMode == null || displayMode == null)
            return;
        Display.displayMode.setDisplayMode(displayMode);
        hasResized = true;
    }

    public static void destroy() {
        glfwDestroyWindow(window);
        glfwTerminate();
    }

    public static long getWindow() {
        return window;
    }

}